######################################################################################
#> BEGIN DESCRIPTION
#> Package: OASIS
#> Type: Package
#> Title: Outlier And Subgroup Identification Statistics
#> Version: 1.1
#> Author: Stan Pounds <stanley.pounds@stjude.org> 
#>            Iwona Pawlikowska <iwona.pawlikowska@stjude.org>
#> Maintainer: Stan Pounds <stanley.pounds@stjude.org>
#> Depends: R (>= 2.15.0), diptest, fdrtool
#> Description: Identifies variables with multimodal distributions (subgroups) or singleton outliers. 
#> License: GPL-2
#> LazyData: yes
#> LazyLoad: yes
#> URL: http://www.stjude.org/pounds  
#> END DESCRIPTION
######################################################################################
#> BEGIN NAMESPACE
#> exportPattern("oasis|row.oasis|pqt|pq.transform|one.oasis.plot|M7example")
#> END NAMESPACE
######################################################################################
#> BEGIN OASIS-package
#> \alias{OASIS-package}
#> \alias{OASIS}
#> \docType{package}
#> \title{OASIS - Outlier And Subgroup Identification Statistics}
#> \description{OASIS identifies variables with multimodal distributions (thus defining 
#> subgroups). It also identifies variables with singleton outliers. OASIS uses four 
#> complimentary statistical methods for these purposes: the max spacing test (Pyke), 
#> the dip test (Hartigan), least median squares outlier detection (Rousseeuw), and 
#> leave-one-out tests. }
#> \details{
#> \tabular{ll}{
#> Package: \tab OASIS\cr
#> Type: \tab Package\cr
#> Version: \tab 1.1\cr
#> Date: \tab 2015-06-03\cr
#> License: \tab GPL(>=2) \cr }
#> }
#> \author{Stan Pounds <stanley.pounds@stjude.org> 
#>         Iwona Pawlikowska <iwona.pawlikowska@stjude.org>
#>
#> Maintainer: Stan Pounds <stanley.pounds@stjude.org>
#> }
#> \references{ Pawlikowska I., Wu G., Edmonson M., Liu Z., Gruber T., Zhang J., Pounds S.,The Most Informative Spacing Test Effectively Discovers Biologically Relevant Outliers or Multiple Modes in Expression. Bioinformatics 2014.}
#>
#>
#> END OASIS-package
########################################################################################
#> BEGIN M7example
#> \alias{M7example}
#> \docType{data}
#> \title{mRNA-seq data for patients with AMKL}
#> \description{mRNA-seq exon read counts for 14 patients with Acute Megakaryoblastic Leukemia (AMKL) treated at St Jude Children's Research Hospital.}
#> \format{ A data frame with 504 observations for the following 18 variables.
#> \describe{
#>           \item{\code{gene}}{gene annotation column}
#>           \item{\code{chrom}}{chromosome column}
#>           \item{\code{loc.start}}{start location of exon}
#>           \item{\code{loc.end}}{end location of exon}
#>           \item{\code{SJAMLM7001.D}}{raw mRNA-seq exon count data for patient 1}
#>           \item{\code{SJAMLM7003.D}}{raw mRNA-seq exon count data for patient 3}
#>           \item{\code{SJAMLM7004.D}}{raw mRNA-seq exon count data for patient 4}
#>           \item{\code{SJAMLM7005.D}}{raw mRNA-seq exon count data for patient 5}
#>           \item{\code{SJAMLM7006.D}}{raw mRNA-seq exon count data for patient 6}
#>           \item{\code{SJAMLM7007.D}}{raw mRNA-seq exon count data for patient 7}
#>           \item{\code{SJAMLM7008.D}}{raw mRNA-seq exon count data for patient 8}
#>           \item{\code{SJAMLM7009.D}}{raw mRNA-seq exon count data for patient 9}
#>           \item{\code{SJAMLM7010.D}}{raw mRNA-seq exon count data for patient 10}
#>           \item{\code{SJAMLM7011.D}}{raw mRNA-seq exon count data for patient 11}
#>           \item{\code{SJAMLM7012.D}}{raw mRNA-seq exon count data for patient 12}
#>           \item{\code{SJAMLM7013.D}}{raw mRNA-seq exon count data for patient 13}
#>           \item{\code{SJAMLM7014.D}}{raw mRNA-seq exon count data for patient 14}
#>           \item{\code{SJAMLM7015.D}}{raw mRNA-seq exon count data for patient 15}}}
#>
#> \examples{
#> data(M7example)
#> head(M7example)}
#> \references{ Gruber TA, et al (2012).  An inv(16)(p13.3;q24.3)-encoded CBFA2T3-GLIS2 fusion protein defines an aggressive subtype of pediatric acute megakaryoblastic leukemia.  \emph{Cancer Cell}, 2012, 22(5): 683-697.}
#>
#> \keyword{datasets}
#> END M7example
#########################################################################################
#> BEGIN oasis
#> \title{OASIS for one variable}
#>
#> \description{Compute Outlier And Subgroup Identification Statistics for One Variable}
#>
#> \arguments{
oasis=function(x,              #> \item{x}{a vector of input data values}
               s0=0,           #> \item{s0}{a constant added to the scale estimate prior to computing the t-statistic. Default is zero.}
               unitize=TRUE,   #> \item{unitize}{If TRUE, x will be shifted and scaled so that its values fall in (0,1) if they don't already.}
               method="all")   #> \item{method}{specify which method to use: "mast" = maximum spacing, "mist"  most informative spacing, "dip" - dip test, "lms.mop" - least median squares with minimum outlier p-value, "lms.sst" - least median squares with sum of squared t-statistics, "loo.mop" - leave-one-out with minimum outlier p-value or "loo.sst" - leave-one-out with sum of squared t-statistics. Default is set to "all".}}
#>
#> \value{a vector with some or all of the following entries, depending on the method(s) chosen:
#>  \item{mast.a}{value of observation that defines the left endpoint of the maximum spacing}
#>  \item{mast.b}{value of observation that defines the right endpoint of the maximum spacing}
#>  \item{mast.stat}{maximum spacing statistic (MAST), the largest first difference of sorted data values.}
#>  \item{p.mast}{p-value for mast statistic}
#>  \item{mast.index}{index of the maximum spacing}
#>  \item{mast.hi}{number of data points located to the right of the maximum spacing}
#>  \item{mist.a}{value of observation that defines the left endpoint of the most informative spacing}
#>  \item{mist.b}{value of observation that defines the right endpoint of the most informative spacing}
#>  \item{mist.stat}{most informative spacing test (MIST) statistic}
#>  \item{mist.index}{index of most informative spacing}
#>  \item{mist.hi}{number of data points located to the right of the most informative spacing}
#>  \item{lms.bulk}{narrowest interval covering half the data}
#>  \item{lms.bulk.index}{indices of sorted data values for the narrowest interval covering half the data}
#>  \item{lms.bulk.size}{width of narrowests interval covering half the data}
#>  \item{lms.center}{least median square (LMS) estimate of center = mean of observations in the narrowest interval covering half the data}
#>  \item{lms.scale}{LMS scale estimate obtained by matching narrowest interval covering half the data to middle 50\% of the normal distribution}
#>  \item{lms.maxt}{maximum outlier t-statistic by LMS}
#>  \item{lms.mop.stat}{minimum outlier p-value by LMS}
#>  \item{lms.sst.stat}{sum of squared outlier t-statistics by LMS}
#>  \item{dip.stat}{the dip statisticof Hartigan and Hartigan}
#>  \item{loo.maxt}{maximum outlier t-statistic by leave-one-out (LOO)}
#>  \item{loo.maxt.index}{index of maximum absolute outlier t-statistic}
#>  \item{loo.mop.stat}{minimum outlier p-value by LOO}
#>  \item{loo.sst.stat}{sum of squared outlier t-statistics by LOO}}
#> 
#> \details{See \code{\link{full.oasis}} for a detailed description of the result statistics.}
#>
#> \examples{
#> u=runif(50)
#> u.res=oasis(u)
#> z=rnorm(50)
#> z.res=oasis(z,unitize=TRUE)
#> b=rbeta(50,0.2,0.2)
#> b.res=oasis(b)
#> }

#> \author{Stan Pounds \email{stanley.pounds@stjude.org}}
{
  # remove missing values from x and sort x if necessary
  na.x=is.na(x)
  x=x[!na.x]
  if (is.unsorted(x)) x=sort(x)
     
  n=length(x)                                                 # sample size
  # unitize x if desired and necessary
  if (unitize) unitize=(x[n]>1)|(x[1]<0)                      # default is to check if data is outside unit interval
  if (unitize)                                                # unitize data if desired
  {
    rng=x[n]-x[1]
    x=(x-x[1]+rng/(n-1))/(rng*(n+1)/(n-1))
  }
  res=NULL
  if (any(is.element(method,c("mist","mast","all"))))
  {
    # compute spacings, max spacing, and most informative spacing
    d=diff(x)                                                            # all spacings
    mast.index=which.max(d)                                              # index of maximum spacing
    mast.a=x[mast.index]                                                #
    mast.b=x[mast.index+1]
    mast.stat=d[mast.index]                                              # maximum spacing
    mast.midpt=mean(x[mast.index+0:1])                                   # midpoint of maximum spacing
    log.p.mast=(n-1)*pbeta(mast.stat,1,length(d)+1,log.p=TRUE,lower.tail=T) # log of p-value for maximum spacing
    p.mast=1-exp(log.p.mast)                                             # p-value for maximum spacing)
    #log.p.mast.f=(n-1)*log(1-(1-mast.stat)^n)
    #p.mast.f=1-exp(log.p.mast.f)
    if (all(d==0)) p.mast=1                                              # p-value is 1 if all data points are equal
    mast.hi.x=(x>=mast.midpt)                                            # number of data points that have bigger values than the maximum spacing  
   
    if (n%%2==0)                                                         # calculate constant k if n is even or odd
      k=((n-1))/sqrt((n+1)*(n-1)) else k=((n-1))/n                   
    info.spacing=d*2*k*sqrt((1:length(d))*(length(d):1))/(length(d))     # information spacing
    mist.index=which.max(info.spacing)                                   # index of most informative spacing
    mist.stat=info.spacing[mist.index]                                   # most informative spacing
    mist.a=x[mist.index]
    mist.b=x[mist.index+1]
    mist.midpt=mean(x[mist.index+0:1])                                   # midpoint of most informative spacing
    mist.hi.x=(x>=mist.midpt)                                            # number of data points that have bigger values than the most informative spacing  
    res=c( res,
           mast.a=mast.a,
           mast.b=mast.b,
           mast.stat=mast.stat,                                          # MAST statistic
           log.p.mast=log.p.mast,
           p.mast=p.mast,                                                # p-value for mast statistic
           mast.index=mast.index,                                        # index of maximum spacing
           mast.hi=n-mast.index,                                         # number of data points that have bigger values than mast   
           mist.a=mist.a,
           mist.b=mist.b,
           mist.stat=mist.stat,                                          # MIST statistic
           mist.index=mist.index,                                        # index of most informative spacing
           mist.hi=n-mist.index)                                         # number of data points that have bigger values than mist
   } 
  
  if (any(is.element(method,c("lms.mop","lms.sst","all"))))
  {
    # compute Least Median Squares
    m=ceiling(n/2)                                                       # ceiling of half the sample size
    hw=diff(x,m-1)                                                       # widths of intervals that cover at least half the data  
    nw.index=which.min(hw)                                               # index of narrowest such interval
    nhw=x[c(nw.index,nw.index+m-1)]                                      # endpoints of the narrowest such interval
    nhw.size=hw[nw.index]                                                # size of the narrowest such interval
    lms.scl=nhw.size/(2*sqrt(qchisq(0.5,1)))                             # scale estimator based on narrowest such interval
    lms.scl=s0+lms.scl                                                   # scale estimator with small factor added
    lms.center=mean(x[nw.index+0:(m-1)])                                 # estimator of center based on narrowest such interval
    lms.tstats=(x-lms.center)/(lms.scl*sqrt(1+1/((n-1)^(2/3))))          # t-statistic based on LMS
    lms.maxt.index=which.max(abs(lms.tstats[c(1,n)]))                    # index of maximum t-statistic; max t-stat for one of the extremes
    lms.maxt=lms.tstats[c(1,n)[lms.maxt.index]]                          # value of maximum t-statistic
    lms.mop=2*pt(-abs(lms.maxt),n-1)                                     # minimum of outlier p-value
    lms.sst=sum(lms.tstats^2)                                            # sum of squared t-statistics
    res=c(res,
          lms.bulk=nhw,                                                  # narrowest interval covering half the data
          lms.bulk.index=c(nw.index,nw.index+m-1),                       # indices of narrowest interval covering half the data
          lms.bulk.size=nhw.size,                                        # width of narrowests interval covering half the data
          lms.center=lms.center,                                         # LMS estimate of center = mean of observations in the narrowest interval covering half the data
          lms.scale=lms.scl,                                             # LMS scale estimate obtained by matching narrowest interval covering half the data to the normal distribution
          lms.maxt=lms.maxt,                                             # maximum outlier t-stat by LMS
          lms.mop.stat=lms.mop,                                          # minimum outlier p-value by LMS
          lms.sst.stat=lms.sst)                                          # sum of squared outlier t-stats by LMS
  }
  
  if (any(is.element(method,c("dip","all"))))
  {
    # Dip test
    dip.stat=dip(x)
    res=c(res,
          dip.stat=dip.stat)                                             # dip statistic
  }
  if (any(is.element(method,c("loo.mop","loo.sst","all"))))
  {
    # LOO
    sum.x=sum(x)                                                         # sum of x for quick computing of LOO stats
    sum.x2=sum(x^2)                                                      # sum of x squared for quick computing of LOO stats
    loo.tstat=rep(NA,n)                                                  # initialize vector for LOO stats
    for (i in 1:n)                                                       # Loop over subjects to leave out
    {
      loo.ss=(sum.x2-x[i]^2)-(sum.x-x[i])^2/(n-1)                        # LOO sum of squares
      loo.mean=(sum.x-x[i])/(n-1)                                        # LOO mean
      loo.sd=sqrt(loo.ss/(n-2))                                          # LOO standard deviation
      loo.scl=s0+loo.sd                                                  # LOO standard deviation with small factor added for numerical stability
      loo.tstat[i]=(x[i]-loo.mean)/(loo.scl*sqrt((1+1/(n-2))))           # LOO outlier t-statistic
    }
    loo.sst=sum(loo.tstat^2)                                             # sum of squared LOO outlier t-stats
    max.tstat.index=which.max(abs(loo.tstat[c(1,n)]))                    # index of maximum absolute outlier t-statistic; max must be one of the extremes
    loo.maxt=loo.tstat[c(1,n)[max.tstat.index]]                          # max LOO t-stat
    loo.mop=2*pt(-abs(loo.maxt),n-2)                                     # min LOO p-value
    res=c(res,
          loo.maxt=loo.maxt,                                             # maximum outlier t-stat by LOO
          loo.maxt.index=max.tstat.index,                                # index of maximum absolute outlier t-statistic
          loo.mop.stat=loo.mop,                                          # minimum outlier p-value by LOO
          loo.sst.stat=loo.sst)                                          # sum of squared outlier t-stats by LOO
  }
  # return results
  return(res)
}
#> END oasis

###################################################################################################
#> BEGIN row.oasis
#> \title{OASIS for each row of a data matrix}
#>
#> \description{Calculate Outlier And Subgroup Identification Statistics for each row of a data set}
#>
#> \arguments{
row.oasis=function(data.set,                       #>  \item{data.set}{a matrix or data.frame.}
                   data.columns=1:ncol(data.set),  #>  \item{data.columns}{columns of data.set to perform OASIS: the index of the columns or the names of the columns.  Default is to include all columns.}
                   s0=1/(2*(nrow(data.set))),      #>  \item{s0}{a constant added to the scale estimate prior to computing the t-statistic. Default is 1/(2*(nrow(data.set))).}
                   unitize=FALSE,                  #>  \item{unitize}{logical, whether to unitize each row of the data.set by separate shift-and-scale transformation. Default is set to FALSE.}
                   method="all",                   #>  \item{method}{specify which method to use: "mast" = maximum spacing, "mist"  most informative spacing, "dip" - dip test, "lms.mop" - least median squares with minimum outlier p-value, "lms.sst" - least median squares with sum of squared t-statistics, "loo.mop" - leave-one-out with minimum outlier p-value or "loo.sst" - leave-one-out with sum of squared t-statistics. Default is set to "all".}
                   sim.stats=NULL,                 #>  \item{sim.stats}{a data.frame with simulated statistics for uniform and normal distributions or NULL. Default is NULL, so the function will calculate simulated statistics.}
                   nsim=10000)                     #>  \item{nsim}{number of simulations for calculating simulated statistics. Default is 10000.}}
#>
#> \value{An object with the similar structure as the results of \link{full.oasis} but includes only the results for the methods indicated by the value of the \code{method} argument.}
#>
#> \references{1.Pawlikowska I., Wu G., Edmonson M., Liu Z., Gruber T., Zhang J., Pounds S. (2014) \emph{Bioinformatics}, in print. \cr
#>             2. Pyke R. (1965) Spacings. \emph{J. R. Statist. Soc. B} (27): 395-449. \cr
#>             3. Rousseeuw P. J. (1984) Least Median of Squares Regression.  \emph{ Journal of the American Statistical Association} (79): 871-880. \cr
#>             4. Hartigan JA and Hartigan PM (1985) The Dip Test of Unimodality, \emph{Annals of Statistics} 13: 70-84. \cr
#>             5. Nord A. S., Lee M., King M.-C., Walsh T. (2011) Accurate and exact CNV identification from targeted high-throughput sequence data.  \emph{BMC Genomics} 12, 184.}
#>
#> \author{Stan Pounds \email{stanley.pounds@stjude.org}}
#>
#>  \examples{
#>  # load data
#>  data(M7example)
#>  # specify columns of data to transform
#>  data.columns=5:18
#>
#>  # perform positive quantile transformation of specified columns of data
#>  pqt.data=pq.transform(M7example,data.columns,add.row.id=TRUE)
#>  # add.row.id=TRUE so there is an additional column with row identifiers
#>  data.columns=data.columns+1  # since there is an additional column row.id
#> 
#>  # define s0
#>  s0=1/(2*(nrow(M7example)))
#>  # perform OASIS and calculate values of simulations statistics
#>  res=row.oasis(pqt.data,data.columns,s0=s0,sim.stats=NULL,nsim=1000)
#>  names(res)
#>  head(res$oasis.res)
#>  head(res$sim.stats)
#> 
#>  # or we can insert values of simulations statistics
#>  res.new=row.oasis(pqt.data,s0=s0,data.columns,sim.stats=res$sim.stats) 
#>  
#>  # we can specify a particular method we are interested in, i.e MIST
#>  res.new=row.oasis(pqt.data,s0=s0,data.columns,sim.stats=res$sim.stats,method="mist")
#>  colnames(res.new$oasis.res) }  
{  
  if (is.null(data.columns)) data.columns=1:ncol(data.set)  
  if (is.null(colnames(data.set))) colnames(data.set)=paste("X",1:ncol(data.set),sep="")
  
  X=data.set[,data.columns]    
  X=as.matrix(X)
  nsubj=ncol(X)                                     # obtain sample size
  ngenes=nrow(X)
  
  if (any(is.character(X))) stop(paste("Some characters in input data. Change data.columns"))
  
  
  if (!is.numeric(s0)) 
  {
    s0=1/(2*ngenes)                                 # define a default value for s0
    warning("s0 should be numeric, using default value of 1/(2*nrow(X))")
  }  
  Y=apply(X,1,sort)                                 # pre-sort each row of data.set for faster computing later on
  Y=as.data.frame(Y)
  
  obs.res=apply(Y,2,oasis,s0=s0,unitize=unitize,method=method)    # obtain oasis stats for pre-sorted data
  if (method=="dip") {
    
    obs.res=as.matrix(obs.res,ncol=1)
    colnames(obs.res)="dip.stat"
  } else {
    obs.res=t(obs.res)                                # transpose results to have desired orientation
    rownames(obs.res)=1:nrow(obs.res)
  }
  
  # combine annotation data (if exists) with oasis statistics
  final.res=cbind.data.frame(data.set[,-data.columns],obs.res) 
  colnames(final.res)=c(colnames(data.set)[-data.columns],
                        colnames(obs.res))
    
  # use simulation to get p-values if desired
  if (is.null(sim.stats)) sim.res=oasis.sim(nsim=nsim,nsubj=nsubj,s0=s0,method=method)
  else sim.res=sim.stats
  
  U.col=grep("usim",colnames(sim.res))
  Z.col=grep("zsim",colnames(sim.res))
    
  U.res=sim.res[,U.col]
  Z.res=sim.res[,Z.col]
  
  if (method=="dip") {
    U.res=as.matrix(U.res,ncol=1)
    Z.res=as.matrix(Z.res,ncol=1)
    colnames(U.res)="dip.stat.usim"
    colnames(Z.res)="dip.stat.zsim"
  }
    
  # identify columns for which to compute simulation-based p-values
  stat.cols=grep("stat",colnames(obs.res))
  test.stats=colnames(obs.res)[stat.cols]
  stat.col.res=rep(NA,length(test.stats))
  stat.col.sim=rep(NA,length(test.stats))
  for (i in 1:length(test.stats)) 
  {
    stat.col.res[i]=grep(paste(test.stats[i],"$",sep=""),colnames(obs.res))
    stat.col.sim[i]=grep(test.stats[i],colnames(U.res),fixed=T)
  }
    
  # compute p-values by comparing sim stats to observed stats
  usim.stats=matrix(NA,ngenes,length(test.stats))
  colnames(usim.stats)=paste(test.stats,".usim.pvalue",sep="")

  pval.tail=rep(1,length(test.stats))
  pval.tail[grep("mop",test.stats)]=-1
  
  for (i in 1:length(test.stats))
    usim.stats[,i]=sim.pvalues(obs.res[,stat.col.res[i]],U.res[,stat.col.sim[i]],pval.tail[i])
  
  # compare sim stats to observed stats to get p-values
  zsim.stats=matrix(NA,ngenes,length(test.stats))
  colnames(zsim.stats)=paste(test.stats,".zsim.pvalue",sep="")

  #print(stat.col.res)
  #print(test.stats)
    
  for (i in 1:length(test.stats)) 
    zsim.stats[,i]=sim.pvalues(obs.res[,stat.col.res[i]],Z.res[,stat.col.sim[i]],pval.tail[i])
    
  # combine oasis statistics with simulated p-values
  final.res=cbind.data.frame(final.res,usim.stats,zsim.stats)
    
  final.res=list(oasis.res=final.res,sim.stats=sim.res)
  
  return(oasis.res=final.res)
  
}

#> END row.oasis

###############################################################
#> BEGIN full.oasis
#> \title{Perform a complete OASIS analysis}
#> \description{Perform a complete OASIS analysis with all methods and save the statistical and graphical results to files.}
#>
#> \arguments{
full.oasis=function(data.set,                      #> \item{data.set}{a data.frame that includes data for OASIS analysis and (optionally) annotations}
                    data.columns=1:ncol(data.set), #> \item{data.columns}{a vector of names or numeric indices of columns that include data for analysis.  Default uses all columns for analysis.  Any other columns in data.set will be appended to the statistical results as annotations.}
                    prep.transform="pqt",          #> \item{prep.transform}{indicates what transformation to use to prepare the data for OASIS analysis: "pqt" for positive quantile transformation, "rng" for unitization of the range, or "none" for none.}
                    s0=1/(2*nrow(data.set)),       #> \item{s0}{a term to add to the LOO and LMS standard deviation estimates to avoid division by zero in computing the LOO and LMS t-statistics.  Defaults to 1/(2*nrow(data.set)).}                       
                    ntop.plot=500,                 #> \item{ntop.plot}{number of top hits to plot for each method, default=500.}
                    plot.title.column=NULL,        #> \item{plot.title.column}{name or numeric index of annotation column with title for plots}
                    output.dir=NULL,               #> \item{output.dir}{name of directory for output files}
                    nsim=10000,                    #> \item{nsim}{number of simulated data sets to generate to compute p-values.}
                    sim.seed=12345                 #> \item{sim.seed}{initial starting seed for simulation, default=12345.}
                    )
#> }

#> \value{A list with components \code{oasis.res} and \code{sim.stats}.  The \code{oasis.res} component is a \code{data.frame} with the following columns (one per row of input data):  
#>  \item{mast.a}{data value that defines the left endpoint of the maximum spacing}
#>  \item{mast.b}{data value that defines the right endpoint of the maximum spacing}
#>  \item{mast.stat}{maximum spacing statistic (MAST)}
#>  \item{p.mast}{p-value for mast statistic}
#>  \item{mast.index}{index of maximum spacing}
#>  \item{mast.hi}{number of data points located to the right of the pair of points defining the MAST}
#>  \item{mist.a}{data value that defines the left endpoint of the most informative spacing}
#>  \item{mist.b}{data value that defines the right endpoint of the most informative spacing}
#>  \item{mist.stat}{most informative spacing test (MIST) statistic}
#>  \item{mist.index}{index of most informative spacing}
#>  \item{mist.hi}{number of data points located to the right of the pair of points defining the MIST }
#>  \item{lms.bulk}{narrowest interval covering half the data}
#>  \item{lms.bulk.index}{indices of sorted data values that define the endpoints of the narrowest interval covering half the data}
#>  \item{lms.bulk.size}{width of narrowests interval covering half the data}
#>  \item{lms.center}{least median square (LMS) estimate of center = mean of observations in the narrowest interval covering half the data}
#>  \item{lms.scale}{LMS scale estimate obtained by matching narrowest interval covering half the data to the first and third quartiles of the normal distribution}
#>  \item{lms.maxt}{maximum outlier t-statistic by LMS}
#>  \item{lms.mop.stat}{minimum outlier p-value by LMS}
#>  \item{lms.sst.stat}{sum of squared outlier t-statistics by LMS}
#>  \item{dip.stat}{dip statistic}
#>  \item{loo.maxt}{maximum outlier t-statistics by leave-one-out (LOO)}
#>  \item{loo.maxt.index}{index of maximum absolute outlier t-statistic}
#>  \item{loo.mop.stat}{minimum outlier p-value by LOO}
#>  \item{loo.sst.stat}{sum of squared outlier t-stats by LOO}
#>  \item{mast.stat.usim.pvalue,mist.stat.usim.pvalue,...}{simulated values for each of the oasis statistic for uniform(0,1) distribution}
#>  \item{mast.stat.zsim.pvalue,mist.stat.zsim.pvalue,...}{simulated values for each of the oasis statistic for normal(0,1) distribution}
#>
#>  "sim.stats" is a data frame with the same number of rows as the data.set and 16 columns
#>  that represent simulated values for each of the oasis statistic for uniform distribution ("usim")
#>  and simulated values for each of the oasis statistic for normal distribution ("zsim"). All the "usim" results on the
#>  same row are results for the same simulated uniform(0,1) data set and all the "zsim" results on the
#>  same row are the results for the same simulated normal(0,1) data set.}

#> \details{The function will apply each test described below to each row of data.
#> 
#>   MIST and MAST:
#>
#>  These tests sort the data values and then compute the differences (spacings)
#>  between consecutive ordered data values. MAST: A very large spacing may indicate an extreme outlier
#>  or correspond to the difference between two very distinct subgroups. The maximum 
#>  spacing is computed and its value is compared to the distribution of the maximum 
#>  spacing of a set of independent uniform(0,1) observations to obtain a p-value. 
#>  MIST: Each spacing is multiplied by a factor that is a function of
#>  its position relative the data values (number of data values to the left and
#>  number of data values to the right) to give spacing information. 
#>  The factor is defined such that it has the largest 
#>  value for spacing dividing two balanced subgroups of data and it has the smallest 
#>  value if the spacing separates largest of smallest observation from the rest of 
#>  the data. The MIST statistic is calculated as the maximum of values of spacing 
#>  across all observations. The observed MIST statistic is compared to its distribution
#>  under the null model that all observations are independent identically distributed
#>  uniform(0,1) and normal(0,1).
#>
#>
#>  LMS:
#>
#>  Each observation is tested for possible outlier using least  median squares (LMS).
#>  LMS identifies the narrowest interval that covers at least 
#>  50\% of the data and assumes that interval defines the ''bulk'' of the 
#>  observations. Next, t-tests are used to evaluate whether each individual 
#>  observation is an outlier relative to a normal distribution with first and 
#>  third quartiles corresponding to the endpoints of the interval. The sum of 
#>  squared t-statistics (SST) and minimum outlier p-value (MOP) are computed to summarize
#>  the results. \eqn{s_0} is a small positive number that the user may add to the LMS estimate of variance 
#>  to avoid a large number ties for biologically not meaningful data (such as many 
#>  zeros in RNA-seq data). Our implementation first determines the empirical null distribution 
#>  for LMS-MOP and LMS-SST by computing the smallest p-value across samples and sum of squared 
#>  t-statistics for each of a large number (default=10,000) of data sets with equal sample size 
#>  generated from the normal(0,1) and uniform(0,1) distributions. Then, the observed MOP of 
#>  each variable  is compared to this empirical null distribution. The p-value is the proportion 
#>  of empirical null MOPs that are greater than or equal to the observed MOP. The p-value
#>  for SST is calculated analougously. By default, the empirical null distributions are 
#>  calculated (sim.stats=NULL).
#>
#>
#>  DIP:
#>
#>  This test evaluates whether the distribution of the data values 
#>  appears multimodal. The dip test compares the observed 
#>  empirical distribution function (EDF) of the data to the unimodal distribution 
#>  function (UDF) that minimizes the maximum difference between the EDF and the UDF.
#>  Hartigan and Hartigan (1985) call this minimax difference the dip statistic.  
#>  Hartigan and Hartigan (1985) prove that the uniform distribution is asymptotically 
#>  the ''least favorable'' unimodal distribution and thus recommend that the test be 
#>  performed by comparing the observed value of the dip statistic to its empirical 
#>  null distributions obtained by generating many data sets of equal size from the 
#>  uniform(0,1) and normal(0,1) distributions. Then, the observed dip statistic of each 
#>  variable is compared to this empirical null distribution.  The p-value is the proportion 
#>  of empirical null dip statistics that are greater than or equal to the observed dip statistic. 
#>  User can insert  empirical null distributions of the dip statistic (default sim.stats=NULL).
#>
#>
#>  LOO:
#>
#>  For each subject i, leave-one-out (LOO) computes the mean and standard deviation of the
#>  remaining observations (with subject i left out). Then, for each subject i, 
#>  an outlier t-statistic and outlier p-value is calculated.
#>  Similarly to LMS, two statistics are calculated to summarize the collection of outlier p-values,
#>  minimum outlier p-value (MOP) and sum of squared t-statistics.
#>  There can be some cases that have large number of ties in the LOO procedure 
#>  (such as many zeros in RNA-seq count data) so that leave-one-out variance estimate 
#>  \eqn{s_i=0} for some subject i and \eqn{|t_i|=\infty}  and \eqn{p_i^{LOO}=0}. Then, these 
#>  features will be extremely significant but usually not of biological interest. To prevent these
#>  technical problems, the user may add a small positive constant \eqn{s_0} to \eqn{s_i}.
#>  The statistical significance of LOO-MOP and LOO-SST is calculated in the similar way to LMS-MOP and LMS-SST.}
{
  # check that prep.transform is OK
  prep.ok=is.element(prep.transform,c("pqt","rng","none"))
  if (!prep.ok) stop("prep.ok must be `pqt' for positive quantile transform, `rng' for unitize the range, or `none' for none.")
  
  # check that output directory is OK
  if (is.null(output.dir)) output.dir=getwd()
  out.dir.exists=file.exists(output.dir)
  out.dir.info=file.info(output.dir)
  if (is.na(out.dir.info$isdir)) out.dir.info$isdir=F
  out.dir.ok=(out.dir.exists&out.dir.info$isdir)
  if (!out.dir.ok) stop("Invalid Output Directory Specified")
  message(paste("Output files will be written to ",output.dir,".",sep=""))
  
  # Check that plot.title.column is OK
  column.names=colnames(data.set)
  if (!is.null(plot.title.column))
  {
    ok.plot.title=F
    if (is.character(plot.title.column)) ok.plot.title=is.element(plot.title.column,column.names)
    if (is.numeric(plot.title.column)) ok.plot.title=is.element(plot.title.column,1:ncol(data.set))
    if (!ok.plot.title) stop("Invalid plot.title.column specified.")
  }
  
  # get columns for analysis
  if (is.character(data.columns))
  {  
    column.names=colnames(data.set)
    data.ind=grep(data.columns,column.names)
  } else data.ind=data.columns
  
  data.ind=grep(data.columns,column.names)
  X=data.set[,data.ind]
  
  # prepare data for analysis, if desired
  if (prep.transform=="pqt")
  {
    X=pq.transform(X,add.row.id=F)
  }
  
  set.seed(sim.seed)
  message(paste("Random seed set to ",sim.seed,".",sep=""))
  oasis.res=row.oasis(X,s0=s0,unitize=(prep.transform=="rng"),
                      nsim=nsim)
  oasis.stats=cbind.data.frame(data.set[,-data.ind], # Annotations
                               oasis.res$oasis.res)  # Statistical Results
  
  oasis.stat.file=paste(output.dir,"/oasis-results.txt",sep="")
  write.table(oasis.stats,oasis.stat.file,sep="\t",col.names=T,row.names=F,quote=T)
  message(paste("Statistical results written to ",oasis.stat.file,".",sep=""))
  
  ntop.plot=min(ntop.plot,nrow(X))
  
  # MAST plots
  if (is.null(plot.title.column)) plot.titles=paste("row",1:nrow(X))
  else plot.titles=data.set[,plot.title.column]
  
  mast.ord=rev(order(oasis.stats$mast.stat))
  mast.plot.file=paste(output.dir,"/top-",ntop.plot,"-mast-plots.pdf",sep="")
  pdf(mast.plot.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[mast.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[mast.ord[i]],
                                        sub=paste("p.MAST (Uniform) =",oasis.stats$p.mast[mast.ord[i]]))
  dev.off()
  message(paste("Plots for top ",ntop.plot," MAST results written to ",mast.plot.file,".",sep=""))
  
  # MIST plots
  mist.ord=rev(order(oasis.stats$mist.stat))
  mist.plot.file=paste(output.dir,"/top-",ntop.plot,"-mist-plots.pdf",sep="")
  pdf(mist.plot.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[mist.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[mist.ord[i]],
                                        sub=paste("p.MIST (Uniform) =",oasis.stats$mist.stat.usim.pvalue[mist.ord[i]],
                                                  "; p.MIST (Normal) =",oasis.stats$mist.stat.zsim.pvalue[mist.ord[i]]))
  dev.off()
  message(paste("Plots for top ",ntop.plot," MIST results written to ",mist.plot.file,".",sep=""))
  
  # LMS-MOP plots
  lms.mop.ord=rev(order(abs(oasis.stats$lms.maxt)))
  lms.mop.file=paste(output.dir,"/top-",ntop.plot,"-lms-mop-plots.pdf",sep="")
  pdf(lms.mop.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[lms.mop.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[lms.mop.ord[i]],
                                        sub=paste("p.LMS.MOP (Uniform) =",oasis.stats$lms.mop.stat.usim.pvalue[lms.mop.ord[i]],
                                                  "; p.LMS.MOP (Normal) =",oasis.stats$lms.mop.stat.zsim.pvalue[lms.mop.ord[i]]))
  dev.off()
  message(paste("Plots for top ",ntop.plot," LMS-MOP results written to ",lms.mop.file,".",sep=""))
  
  # LMS-SST plots
  lms.sst.ord=rev(order(abs(oasis.stats$lms.sst.stat)))
  lms.sst.file=paste(output.dir,"/top-",ntop.plot,"-lms-sst-plots.pdf",sep="")
  pdf(lms.sst.file)
  for (i in 1:ntop.plot)    one.oasis.plot(X[lms.sst.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                           unitize=(prep.transform=="rng"),
                                           main=plot.titles[lms.sst.ord[i]],
                                           sub=paste("p.LMS.SST (Uniform) =",oasis.stats$lms.sst.stat.usim.pvalue[lms.sst.ord[i]],
                                                     "; p.LMS.SST (Normal) =",oasis.stats$lms.sst.stat.zsim.pvalue[lms.sst.ord[i]]))
  dev.off()
  message(paste("Plots for top ",ntop.plot," LMS-SST results written to ",lms.sst.file,".",sep=""))
  
  # LOO-MOP plots
  loo.mop.ord=rev(order(abs(oasis.stats$loo.maxt)))
  loo.mop.file=paste(output.dir,"/top-",ntop.plot,"-loo-mop-plots.pdf",sep="")
  pdf(loo.mop.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[loo.mop.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[loo.mop.ord[i]],
                                        sub=paste("p.LOO.MOP (Uniform) =",oasis.stats$loo.mop.stat.usim.pvalue[loo.mop.ord[i]],
                                                  "; p.LOO.MOP (Normal) =",oasis.stats$loo.mop.stat.zsim.pvalue[loo.mop.ord[i]]))

  dev.off()
  message(paste("Plots for top ",ntop.plot," LOO-MOP results written to ",loo.mop.file,".",sep=""))
  
  # LOO-SST plots
  loo.sst.ord=rev(order(abs(oasis.stats$loo.sst.stat)))
  loo.sst.file=paste(output.dir,"/top-",ntop.plot,"-loo-sst-plots.pdf",sep="")
  pdf(loo.sst.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[loo.sst.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[loo.sst.ord[i]],
                                        sub=paste("p.LOO.SST (Uniform) =",oasis.stats$loo.sst.stat.usim.pvalue[loo.sst.ord[i]],
                                                  "; p.LOO.SST (Normal) =",oasis.stats$loo.sst.stat.zsim.pvalue[loo.sst.ord[i]]))

  dev.off()
  message(paste("Plots for top ",ntop.plot," LOO-SST results written to ",loo.sst.file,".",sep="")) 
  
  # DIP test plots
  dip.ord=rev(order(oasis.stats$dip.stat))
  dip.file=paste(output.dir,"/top-",ntop.plot,"-dip-test-plots.pdf",sep="")
  pdf(dip.file)
  for (i in 1:ntop.plot) one.oasis.plot(X[dip.ord[i],],sim.stats=oasis.res$sim.stats,s0=s0,
                                        unitize=(prep.transform=="rng"),
                                        main=plot.titles[dip.ord[i]],
                                        sub=paste("p.DIP (Uniform) =",oasis.stats$dip.stat.usim.pvalue[dip.ord[i]],
                                                  "; p.DIP (Normal) =",oasis.stats$dip.stat.zsim.pvalue[dip.ord[i]]))

  dev.off()
  message(paste("Plots for top ",ntop.plot," dip test results written to ",loo.sst.file,".",sep="")) 
  
  
  
  
  return(oasis.res)
  

  
}

#> \examples{
#> ## Not run
#> # data(M7example)
#> # head(M7example)
#> # M7.pqt=pq.transform(M7example,5:18,F)
#> # head(M7.pqt)
#> # summary(M7.pqt)
#> # output.folder=getwd()   # set the output folder
#> # M7.res=full.oasis(M7example,
#> #                   data.columns=5:18,
#> #                   output.dir=output.folder,
#> #                   plot.title.column=1,
#> #                   ntop.plot=50,
#> #                   nsim=100)
#> # names(M7.res)
#> # head(M7.res$oasis.res)
#> # head(M7.res$sim.stats)
#> }

#> END full.oasis


################################################################################################
#> BEGIN oasis.sim
#> \title{Simulation statistics for OASIS}
#>
#> \description{Calculate simulations statistics for all OASIS methods}
#>
#> \arguments{
oasis.sim=function(nsim=10000,         #>  \item{nsim}{number of simulations, default is 10000.}
                   nsubj,              #>  \item{nsubj}{number of subjects}
				           s0=0,               #>  \item{s0}{a constant added to the scale estimate prior to computing the t-statistic. Default is zero.} 
                   method="all")       #>  \item{method}{specify which method to use: "mast" = maximum spacing, "mist"  most informative spacing, "dip" - dip test, "lms.mop" - least median squares with minimum outlier p-value, "lms.sst" - least median squares with sum of squared t-statistics, "loo.mop" - leave-one-out with minimum outlier p-value or "loo.sst" - leave-one-out with sum of squared t-statistics. Default is set to "all".}}
                   
#>
#> \value{a data frame with the following columns (in the case method="all"):
#> \item{mast.stat.usim}{MAST statistic for a data set simulated from the uniform(0,1) distribution.}
#> \item{mist.stat.usim}{MIST statistic for a data set simulated from the uniform(0,1) distribution.}
#> \item{lms.mop.usim}{least median squares minimum outlier p-value for a data set simulated from the uniform(0,1) distribution.}
#> \item{lms.sst.usim}{least median squares sum of squared t-statistics for a data set simulated from the uniform(0,1) distribution.}
#> \item{dip.stat.usim}{the dip statistic for a data set simulated from the uniform(0,1) distribution}
#> \item{loo.mop.usim}{the leave-one-out minimum outlier p-value for a data set simulated from the uniform(0,1) distribution.}
#> \item{loo.sst.usim}{the leave-one-out sum of squared t-statistics for a data set simulated from the uniform(0,1) distribution.}
#> \item{mast.stat.zsim}{MAST statistic for a data set simulated from the normal(0,1) distribution.}
#> \item{mist.stat.zsim}{MIST statistic for a data set simulated from the normal(0,1) distribution.}
#> \item{lms.mop.stat.zsim}{least median squares minimum outlier p-value for a data set simulated from the normal(0,1) distribution.}
#> \item{lms.sst.stat.zsim}{least median squares sum of squared t-statistics for a data set simulated from the normal(0,1) distribution.}
#> \item{dip.stat.zsim}{the dip statistic for a data set simulated from the normal(0,1) distribution}
#> \item{loo.mop.stat.zsim}{the leave-one-out minimum outlier p-value for a data set simulated from the normal(0,1) distribution.}
#> \item{loo.sst.stat.zsim}{the leave-one-out sum of squared t-statistics for a data set simulated from the normal(0,1) distribution.}
#> See \code{\link{full.oasis}} for more description of theses statistics.  All the "usim" results on the
#> same row are results for the same simulated uniform(0,1) data set and all the "zsim" results on the
#> same row are the results for the same simulated normal(0,1) data set.}
#> 
#> \examples{
#> sim.res=oasis.sim(nsim=1000,nsubj=50)
#> head(sim.res)}
#> \author{Stan Pounds \email{stanley.pounds@stjude.org}}
{
    
  U=matrix(runif(nsubj*nsim),nsim,nsubj)                                 # simulate nsim data sets of size n from uniform(0,1)
  U.res=apply(U,1,oasis,s0=s0,method=method)                             # obtain oasis statistics for each simulated data set
  Z=matrix(rnorm(nsubj*nsim),nsim,nsubj)                                 # simulate nsim data sets of size n from normal(0,1)
  Z.res=apply(Z,1,oasis,s0=s0,method=method)                             # obtain oasis statistics for each simulated data set
  if (method=="dip") {
    
    res=cbind.data.frame(U.res,Z.res)
    colnames(res)=paste("dip.stat",c(".usim",".zsim"),sep="")
    return(res)
    
  } else {
    U.res=t(U.res)
    stat.col=grep("stat",colnames(U.res))
    U.res=U.res[,stat.col]
    colnames(U.res)=paste(colnames(U.res),".usim",sep="")    
 
    Z.res=t(Z.res)
    stat.col=grep("stat",colnames(Z.res))
    Z.res=Z.res[,stat.col]
    colnames(Z.res)=paste(colnames(Z.res),".zsim",sep="") 
    res=cbind.data.frame(U.res,Z.res)
    return(res)   
  }  
}

#> END oasis.sim

################################################################################################
# Function  sim.pvalues
# Purpose   compute simulation p-values by comparing observed stats to sim stats
# Arguments obs.stats  -  vector of observed statistics
#           sim.stats  -  vector of statistics obtained via simulation
#           tail       -  tail=2 for two-sided, tail=1 for right-sided, tail=-1 for left-sided
# Values    vector of simulated p-values
################################################################################################
sim.pvalues=function(obs.stats, 
                     sim.stats,  
                     tail=2)    
{
  if (tail==2)  # transform stats by absolute value for two-tailed test
  {
    obs.stats=abs(obs.stats)
    sim.stats=abs(sim.stats)
  }
  if (tail==-1) # change sign of stats to compute p-values in left-tail
  {
    obs.stats=-obs.stats
    sim.stats=-sim.stats
  }
  # create a joint data frame with observed and simulated stats
  comb.stats=cbind(index=c(rep(0,length(sim.stats)),1:length(obs.stats)), # assign simulated stats the index 0; assign observed stats their entry number
                   stat=c(sim.stats,obs.stats),                           # 
                   p.value=rep(NA,length(sim.stats)+length(obs.stats)))   # initialize all p-values to NA
  ord=rev(order(!is.na(comb.stats[,2]),comb.stats[,2],-comb.stats[,1]))                          # order by value of statistic and then by whether stat is observed or simulated to propery handle ties
  comb.stats=comb.stats[ord,]
  comb.stats[,3]=cumsum(comb.stats[,1]==0)/length(sim.stats)              # use ordered data frame to compute p-value as a function of the statistic
  keep.rows=which(comb.stats[,1]!=0)                                      # now remove the simulated stats from the object
  if (length(keep.rows)==1) return(comb.stats[keep.rows,3])               # return p-value if only one observed statistic was provided
  comb.stats=comb.stats[keep.rows,]                                       # otherwise return vector with p-values in original order
  ord=order(comb.stats[,1])
  comb.stats=comb.stats[ord,]
  return(comb.stats[,3])
}

################################################################################################
# Function  fit.udf
# Purpose   Fit the non-parametric unimodal distribution function to a set of univariate observations
# Arguments x - vector of data values
# Values    
################################################################################################

fit.udf=function(x,rm.na=TRUE)
{
  na.x=is.na(x)
  if ((!rm.na)&any(na.x)) stop("NAs in x")
  x=x[!na.x]
  x=sort(x)
  
  y1=rank(x,ties.method="max")/length(x)
  y0=rank(x,ties.method="min")/length(x)
  y=rank(x,ties.method="first")/length(x)
  
  dup.x=duplicated(x)
  x.uniq=x[!dup.x]
  y.top=y1[!dup.x]
  y.bot=y0[!dup.x]
  y.bot=c(0,y.top[-length(y.top)])
  
  dip.res=dip(x,full.result=T)
  
  lo=(x.uniq<=dip.res$xl)
  if (sum(lo)==1) udf.gcm=cbind.data.frame(x=x.uniq[lo],y=y.bot[lo])
  else
  {
    gcm.res=gcmlcm(x.uniq[lo],y.bot[lo],"gcm")
    udf.gcm=cbind.data.frame(x=gcm.res$x.knots,y=gcm.res$y.knots)
  }
  
  hi=(x.uniq>=dip.res$xu)
  if (sum(hi)==1) udf.lcm=cbind.data.frame(x=x.uniq[hi],y=y.top[hi])
  else
  {
    lcm.res=gcmlcm(x.uniq[hi],y.top[hi],"lcm")
    udf.lcm=cbind.data.frame(x=lcm.res$x.knots,y=lcm.res$y.knots)
  }
  
  
  udf=rbind.data.frame(udf.gcm,udf.lcm)
  
  edf=cbind.data.frame(x=rep(x.uniq,each=2),
                       y.edf=sort(c(y.bot,y.top)))
  
  edf$y.udf=NA
  edf$y.udf=approx(udf$x,
                   udf$y,ties="ordered",xout=edf$x,
                   yleft=0,yright=1)$y
  
  # 
  dup.udf.x=duplicated(udf$x)
  dup.udf.xvals=udf$x[dup.udf.x]
  need.to.adjust=((1:nrow(edf))%%2==1)&(is.element(edf$x,dup.udf.xvals))
  
  edf$y.udf[need.to.adjust]=approx(udf$x[!dup.udf.x],
                                   udf$y[!dup.udf.x],
                                   ties="ordered",
                                   xout=edf$x[need.to.adjust])$y
  
  
  edf$abs.diff=abs(edf$y.edf-edf$y.udf)
  
  dip.index=which.max(edf$abs.diff)
  x.dip=edf$x[dip.index]
  udf.dip=edf$y.udf[dip.index]
  edf.dip=edf$y.edf[dip.index]
  
  
  return(list(x=x,y=y,
              udf=udf,
              edf=edf,
              x.dip=x.dip,
              udf.dip=udf.dip,
              edf.dip=edf.dip,
              dip.stat=dip.res$dip))
}

##################################################################################################
#> BEGIN one.oasis.plot
#>
#> \title{Plotting OASIS results}
#>
#> \description{Plots data values and confidence intervals for all OASIS methods for a vector of data values}
#>
#> \arguments{
one.oasis.plot=function(x,                      #> \item{x}{a vector of pqt transformed data values}
                        s0=0,                   #> \item{s0}{a constant added to the scale estimate prior to computing the t-statistic for LMS and LOO. Default is 1/(2*nrow(data.set)).}
                        unitize=FALSE,          #> \item{unitize}{logical, whether to unitize x. Default is NULL.}
                        alpha=0.01,             #> \item{alpha}{significance level for confidence intervals}
                        nsim=ceiling(10/alpha), #> \item{nsim}{number of simulations for calculating p-values. Default is 10000.}
                        sim.stats=NULL,         #> \item{sim.stats}{list with two matrices with simulated statistics for uniform and normal distributions, default is null, so the function will calculate the simulated values.}
                        ...)                    #> \item{...}{other graphical options for \code{plot}, such as \code{main}, etc.}}
#>
#> \value{NULL}
#>  \author{Stan Pounds \email{stanley.pounds@stjude.org}}
#>
#> \examples{
#> u=runif(50)
#> one.oasis.plot(u)
#> z=rnorm(50)
#> one.oasis.plot(z,unitize=TRUE)
#> 
#> # make a plot of an exon of the GLIS2 gene that was
#> # found to have bimodal distribution (cf. Gruber et al. 2012)
#> data.columns=5:18
#> # perform positive quantile transformation of specified columns of data
#> pqt.data=pq.transform(M7example,data.columns,add.row.id=TRUE)
#> glis2=grep("GLIS2",pqt.data$gene)
#> x=pqt.data[glis2[8],data.columns+1] # since the first column is added (row.id)
#> one.oasis.plot(as.numeric(x))}
{
  # remove missing values from x and sort x if necessary
  x0=x
  na.x=is.na(x)
  x=x[!na.x]
  if (is.unsorted(x)) x=sort(x) 
  n=length(x)                                                          # sample size
  
  # unitize x if desired or necessary
  if (unitize) unitize=(x[n]>1)|(x[1]<0)                      # default is to check if data is outside unit interval
  if (unitize)                                                # unitize data if desired
  {
    rng=x[n]-x[1]
    x=(x-x[1]+rng/(n-1))/(rng*(n+1)/(n-1))
  }
  
  mist.seg.y=-0.15
  mast.seg.y=-0.20
  lms.seg.y=-0.25
  loo.seg.y=-0.30
  

  if (is.null(sim.stats)){
    sim.res=oasis.sim(nsim=nsim,nsubj=n,s0 = s0)
    sim.stats=sim.res
  }
  #print(names(sim.stats))
  #print(colnames(sim.stats))
  oasis.res=oasis(x,s0=s0,unitize=unitize,method="all")
  u.sim=grep("usim",colnames(sim.stats))
  U.res=sim.stats[,u.sim]
  test.stats=c("mist.stat","dip.stat")
  stat.col.sim=rep(NA,length(test.stats))
  for (i in 1:length(test.stats)) 
    stat.col.sim[i]=grep(paste(test.stats[i],"usim",sep="."),colnames(U.res),fixed=T)
  
  usim.cut=rep(NA,length(test.stats))
  names(usim.cut)=paste(test.stats,".cut",sep="")
    
  for (i in 1:length(test.stats))
    usim.cut[i]=quantile(U.res[,stat.col.sim[i]],1-alpha)
  
  
  max.out=as.numeric(oasis.res["loo.maxt.index"])
  loo.mean=mean(as.numeric(x0[-max.out]))
  loo.sd=sd(x0[-max.out])+s0
  loo.z=loo.mean+c(-1,+1)*loo.sd*sqrt(1+1/(n-2))*qt(1-alpha/2,n-2) 
  loo.z[loo.z<=0]=0
  loo.z[loo.z>=1]=1
  loo.neigh=(x<loo.z[1])|(x>loo.z[2])
  
  
  lms.mean=oasis.res["lms.center"]
  lms.sd=oasis.res["lms.scale"]
  lms.z=lms.mean+c(-1,+1)*lms.sd*sqrt(1+1/((n-1)^(2/3)))*qt(1-alpha/2,n-1)
  lms.z[lms.z<=0]=0
  lms.z[lms.z>=1]=1
  lms.neigh=(x<lms.z[1])|(x>lms.z[2])
  
    
    
  mast.q=qbeta((1-alpha)^(1/(n-1)),1,n)
  mast.indx=oasis.res["mast.index"]
  mast.x1=x[mast.indx]
  mast.x2=mast.x1+mast.q
  mast.y=mast.indx/n
  mast.xplot=c(mast.x1,mast.x2,mast.x2,mast.x1,mast.x1)
  mast.yplot=c(mast.y,mast.y,mast.y+1/n,mast.y+1/n,mast.y)
  mast.pt1=mast.x1
  mast.pt2=x[mast.indx+1]
  
 
  
  mist.indx=oasis.res["mist.index"]
  mist.x1=x[mist.indx]
  mist.x2=mist.x1+usim.cut["mist.stat.cut"]
  mist.y=mist.indx/n
  mist.xplot=c(mist.x1,mist.x2,mist.x2,mist.x1,mist.x1)
  mist.yplot=c(mist.y,mist.y,mist.y+1/n,mist.y+1/n,mist.y)
  mist.pt1=mist.x1
  mist.pt2=x[mist.indx+1]
  
  
  x.rng=range(x,lms.z,mast.x1,mast.x2,mist.x1,mist.x2)
  x.rng[x.rng<0]=0
  plot(c(-0.1,1),c(loo.seg.y,1),type="n",ylab="EDF",xlab="Data Values",...,ylim=c(-0.3,(n+1)/n),xlim=c(-0.15,1.15),axes=F)
  rect(-0.05,-0.05,1.05,1.05)
#   text(1.01,c(mist.seg.y,mast.seg.y,lms.seg.y,loo.seg.y)+0.01,
#        c("MIST","MAST","LMS","LOO"),col=c("orange","red","purple","cyan"),adj=0,cex=0.75)
  text(0,c(mist.seg.y,mast.seg.y,lms.seg.y,loo.seg.y)+0.01,
       c("MIST","MAST","LMS-SST","LOO-SST"),col=c("orange","red","purple","cyan"),pos=2,cex=0.75)
  pvals=c(p.mist=mean(sim.stats[,"mist.stat.usim"]>=oasis.res["mist.stat"]),
          p.mast=oasis.res["p.mast"],
          p.lms=mean(sim.stats[,"lms.sst.stat.zsim"]>=oasis.res["lms.sst.stat"]),
          p.loo=mean(sim.stats[,"loo.sst.stat.zsim"]>=oasis.res["loo.sst.stat"]))
  text(1,c(mist.seg.y,mast.seg.y,lms.seg.y,loo.seg.y)+0.01,
       paste("p=",round(pvals,log10(nrow(sim.stats))),sep=""),
       col=c("orange","red","purple","cyan"),pos=4,cex=0.75)
  
  
  segments(min(lms.z),lms.seg.y,max(lms.z),lms.seg.y,col="purple",lwd=2)
  segments(range(lms.z),lms.seg.y-1/(4*n),range(lms.z),lms.seg.y+1/(4*n),lwd=2,col="purple")
  
  segments(min(loo.z),loo.seg.y,max(loo.z),loo.seg.y,col="cyan",lwd=2)
  segments(range(loo.z),loo.seg.y-0.5/(2*n),range(loo.z),loo.seg.y+0.5/(2*n),lwd=2,col="cyan")
  
  mdpt=mean(x[mast.indx+0:1])
  mast.range=mdpt+c(-1,1)*mast.q/2
  if (mast.range[1]<0) mast.range[1]=0
  if (mast.range[2]>1) mast.range[2]=1
  segments(mast.range[1],mast.seg.y,mast.range[2],mast.seg.y,col="red",lwd=2)
  segments(mast.range,mast.seg.y-1/(4*n),mast.range,mast.seg.y+1/(4*n),col="red",lwd=2)
  points(c(mast.pt1,mast.pt2),rep(mast.seg.y,2),col="red",pch=19)
  
  mist.mdpt=mean(x[mist.indx+0:1])
  mist.range=mist.mdpt+c(-1,1)*usim.cut["mist.stat.cut"]/2
  if (mist.range[1]<0) mist.range[1]=0
  if (mist.range[2]>1) mist.range[2]=1
  segments(mist.range[1],mist.seg.y,mist.range[2],mist.seg.y,col="orange",lwd=2)
  segments(mist.range,mist.seg.y-1/(4*n),mist.range,mist.seg.y+1/(4*n),col="orange",lwd=2)
  points(c(mist.pt1,mist.pt2),rep(mist.seg.y,2),col="orange",pch=19)
  
  dip.res=dip(x,full.result=T)
  udf.res=fit.udf(x)
  
  y1=udf.res$udf.dip
  y2=udf.res$edf.dip
   
  points(c(1.1,1.1),c(y1,y2),col="blue",pch=19)
  dip.sim.bars=(y1+y2)/2+c(-1,1)*usim.cut["dip.stat.cut"]
  dip.sim.bars=dip.sim.bars
  if (dip.sim.bars[2]>1) dip.sim.bars[2]=1
  if (dip.sim.bars[1]<0) dip.sim.bars[1]=0
  dip.pval=mean(sim.stats[,"dip.stat.usim"]>=oasis.res["dip.stat"])
  text(1.12,mean(dip.sim.bars),
       paste("DIP p=",round(dip.pval,log10(nrow(sim.stats))),sep=""),
       srt=90,
       cex=0.75,col="blue")
  
  segments(1.09,dip.sim.bars,1.11,col="blue",lwd=2)
  segments(1.1,dip.sim.bars[1],1.1,dip.sim.bars[2],col="blue",lwd=2)
  lines(udf.res$edf$x,udf.res$edf$y.udf,col="darkgray",type="b")
  
  segments(-0.05,0:5/5,-0.06)
  text(-0.1,0:5/5,0:5/5)
  
  segments(0:5/5,-0.05,0:5/5,-0.06)
  text(0:5/5,-0.10,0:5/5)
  
  points(x,1:n/n,pch=19)
  points(x[lms.neigh],rep(lms.seg.y,sum(lms.neigh)),pch=19,col="purple")
  points(x[loo.neigh],rep(loo.seg.y,sum(loo.neigh)),pch=19,col="cyan")
  
 }
#> END one.oasis.plot

#################################################################################################
#> BEGIN pq.transform
#>
#> \title{Positive Quantile Transformation}
#>
#> \description{Apply Positive Quantile Transformation to each column of a data set} 
#>
#> \arguments{
pq.transform=function(data.set,                       #> \item{data.set}{a data.frame or name of a tab-delimited data file}
                      data.columns=colnames(data.set),#> \item{data.columns}{columns of data.set to transform}
                      add.row.id=TRUE)                #> \item{add.row.id}{ logical, indicates whether to add a columns with row identifiers to the result, default=TRUE.}}
  #>
  #>\value{a data.frame with the pqt tranformed values substituted for data.columns}
  #>
  #> \details{For each subject, the raw expression values are normalized by determining 
  #>  their quantile against the positive raw expression values.
  #>  If data.columns is NULL, then all columns of data.set will be transformed.
  #>  If data.columns is a character vector, then all columns of data.set with a name found in
  #>  data.columns will be transformed.  If data.columns is a numeric vector, then 
  #>  the columns with those numeric indices will be transformed.}
#> 
#>  \author{Stan Pounds \email{stanley.pounds@stjude.org}}
#>
#> \examples{
#>
#> # load data
#>  data(M7example)
#>  
#>  # specify columns of data to transform
#>  data.columns=5:18
#>  
#>  # perform positive quantile transformation of specified columns of data
#>  pqt.data=pq.transform(M7example,data.columns,add.row.id=TRUE)}
#>
{
  
  if (is.character(data.set)) data.set=read.table(data.set,sep="\t",as.is=TRUE,header=TRUE)
  
  if (is.character(data.columns))
  {  
    column.names=colnames(data.set)
    data.columns=grep(data.columns,column.names)
  } 
  
  X=as.matrix(data.set[,data.columns])
  Y=apply(X,2,pqt)
  data.set[,data.columns]=Y
  if (add.row.id) data.set=cbind.data.frame(row.id=1:nrow(data.set),data.set)
  return(data.set)
}

#> END pq.transform


#####################################################################################
#> BEGIN  pqt
#> \title{Positive Quantile Transformation}
#> 
#> \description{Positive Quantile Transformation of a vector of data values}
#>
#> \arguments{
pqt=function(x) #> \item{x}{vector of data values}}
#>
#> \value{
#>  \item{}{vector of pqt data}}
#>   
#> \seealso{\code{\link{pq.transform}}}
{
  x1<-x[x>0]
  x1.tab<-table(x1)
  x1.cdf<-cumsum(x1.tab)/sum(x1.tab)
  res<-approx(c(0,as.numeric(names(x1.tab))),c(0,x1.cdf),xout=x,rule=2)$y
  return(res)
}

#> END pqt