\name{full.oasis}
\alias{full.oasis}
 \title{Perform a complete OASIS analysis}
 \description{Perform a complete OASIS analysis with all methods and save the statistical and graphical results to files.}

\usage{
full.oasis (data.set, data.columns = 1:ncol(data.set), prep.transform = "pqt", 
    s0 = 1/(2 * nrow(data.set)), ntop.plot = 500, plot.title.column = NULL, 
    output.dir = NULL, nsim = 10000, sim.seed = 12345) 
}
 \arguments{
 \item{data.set}{a data.frame that includes data for OASIS analysis and (optionally) annotations}
 \item{data.columns}{a vector of names or numeric indices of columns that include data for analysis.  Default uses all columns for analysis.  Any other columns in data.set will be appended to the statistical results as annotations.}
 \item{prep.transform}{indicates what transformation to use to prepare the data for OASIS analysis: "pqt" for positive quantile transformation, "rng" for unitization of the range, or "none" for none.}
 \item{s0}{a term to add to the LOO and LMS standard deviation estimates to avoid division by zero in computing the LOO and LMS t-statistics.  Defaults to 1/(2*nrow(data.set)).}                       
 \item{ntop.plot}{number of top hits to plot for each method, default=500.}
 \item{plot.title.column}{name or numeric index of annotation column with title for plots}
 \item{output.dir}{name of directory for output files}
 \item{nsim}{number of simulated data sets to generate to compute p-values.}
 \item{sim.seed}{initial starting seed for simulation, default=12345.}
 }
 \value{A list with components \code{oasis.res} and \code{sim.stats}.  The \code{oasis.res} component is a \code{data.frame} with the following columns (one per row of input data):  
  \item{mast.a}{data value that defines the left endpoint of the maximum spacing}
  \item{mast.b}{data value that defines the right endpoint of the maximum spacing}
  \item{mast.stat}{maximum spacing statistic (MAST)}
  \item{p.mast}{p-value for mast statistic}
  \item{mast.index}{index of maximum spacing}
  \item{mast.hi}{number of data points located to the right of the pair of points defining the MAST}
  \item{mist.a}{data value that defines the left endpoint of the most informative spacing}
  \item{mist.b}{data value that defines the right endpoint of the most informative spacing}
  \item{mist.stat}{most informative spacing test (MIST) statistic}
  \item{mist.index}{index of most informative spacing}
  \item{mist.hi}{number of data points located to the right of the pair of points defining the MIST }
  \item{lms.bulk}{narrowest interval covering half the data}
  \item{lms.bulk.index}{indices of sorted data values that define the endpoints of the narrowest interval covering half the data}
  \item{lms.bulk.size}{width of narrowests interval covering half the data}
  \item{lms.center}{least median square (LMS) estimate of center = mean of observations in the narrowest interval covering half the data}
  \item{lms.scale}{LMS scale estimate obtained by matching narrowest interval covering half the data to the first and third quartiles of the normal distribution}
  \item{lms.maxt}{maximum outlier t-statistic by LMS}
  \item{lms.mop.stat}{minimum outlier p-value by LMS}
  \item{lms.sst.stat}{sum of squared outlier t-statistics by LMS}
  \item{dip.stat}{dip statistic}
  \item{loo.maxt}{maximum outlier t-statistics by leave-one-out (LOO)}
  \item{loo.maxt.index}{index of maximum absolute outlier t-statistic}
  \item{loo.mop.stat}{minimum outlier p-value by LOO}
  \item{loo.sst.stat}{sum of squared outlier t-stats by LOO}
  \item{mast.stat.usim.pvalue,mist.stat.usim.pvalue,...}{simulated values for each of the oasis statistic for uniform(0,1) distribution}
  \item{mast.stat.zsim.pvalue,mist.stat.zsim.pvalue,...}{simulated values for each of the oasis statistic for normal(0,1) distribution}

  "sim.stats" is a data frame with the same number of rows as the data.set and 16 columns
  that represent simulated values for each of the oasis statistic for uniform distribution ("usim")
  and simulated values for each of the oasis statistic for normal distribution ("zsim"). All the "usim" results on the
  same row are results for the same simulated uniform(0,1) data set and all the "zsim" results on the
  same row are the results for the same simulated normal(0,1) data set.}
 \details{The function will apply each test described below to each row of data.
 
   MIST and MAST:

  These tests sort the data values and then compute the differences (spacings)
  between consecutive ordered data values. MAST: A very large spacing may indicate an extreme outlier
  or correspond to the difference between two very distinct subgroups. The maximum 
  spacing is computed and its value is compared to the distribution of the maximum 
  spacing of a set of independent uniform(0,1) observations to obtain a p-value. 
  MIST: Each spacing is multiplied by a factor that is a function of
  its position relative the data values (number of data values to the left and
  number of data values to the right) to give spacing information. 
  The factor is defined such that it has the largest 
  value for spacing dividing two balanced subgroups of data and it has the smallest 
  value if the spacing separates largest of smallest observation from the rest of 
  the data. The MIST statistic is calculated as the maximum of values of spacing 
  across all observations. The observed MIST statistic is compared to its distribution
  under the null model that all observations are independent identically distributed
  uniform(0,1) and normal(0,1).


  LMS:

  Each observation is tested for possible outlier using least  median squares (LMS).
  LMS identifies the narrowest interval that covers at least 
  50\% of the data and assumes that interval defines the ''bulk'' of the 
  observations. Next, t-tests are used to evaluate whether each individual 
  observation is an outlier relative to a normal distribution with first and 
  third quartiles corresponding to the endpoints of the interval. The sum of 
  squared t-statistics (SST) and minimum outlier p-value (MOP) are computed to summarize
  the results. \eqn{s_0} is a small positive number that the user may add to the LMS estimate of variance 
  to avoid a large number ties for biologically not meaningful data (such as many 
  zeros in RNA-seq data). Our implementation first determines the empirical null distribution 
  for LMS-MOP and LMS-SST by computing the smallest p-value across samples and sum of squared 
  t-statistics for each of a large number (default=10,000) of data sets with equal sample size 
  generated from the normal(0,1) and uniform(0,1) distributions. Then, the observed MOP of 
  each variable  is compared to this empirical null distribution. The p-value is the proportion 
  of empirical null MOPs that are greater than or equal to the observed MOP. The p-value
  for SST is calculated analougously. By default, the empirical null distributions are 
  calculated (sim.stats=NULL).


  DIP:

  This test evaluates whether the distribution of the data values 
  appears multimodal. The dip test compares the observed 
  empirical distribution function (EDF) of the data to the unimodal distribution 
  function (UDF) that minimizes the maximum difference between the EDF and the UDF.
  Hartigan and Hartigan (1985) call this minimax difference the dip statistic.  
  Hartigan and Hartigan (1985) prove that the uniform distribution is asymptotically 
  the ''least favorable'' unimodal distribution and thus recommend that the test be 
  performed by comparing the observed value of the dip statistic to its empirical 
  null distributions obtained by generating many data sets of equal size from the 
  uniform(0,1) and normal(0,1) distributions. Then, the observed dip statistic of each 
  variable is compared to this empirical null distribution.  The p-value is the proportion 
  of empirical null dip statistics that are greater than or equal to the observed dip statistic. 
  User can insert  empirical null distributions of the dip statistic (default sim.stats=NULL).


  LOO:

  For each subject i, leave-one-out (LOO) computes the mean and standard deviation of the
  remaining observations (with subject i left out). Then, for each subject i, 
  an outlier t-statistic and outlier p-value is calculated.
  Similarly to LMS, two statistics are calculated to summarize the collection of outlier p-values,
  minimum outlier p-value (MOP) and sum of squared t-statistics.
  There can be some cases that have large number of ties in the LOO procedure 
  (such as many zeros in RNA-seq count data) so that leave-one-out variance estimate 
  \eqn{s_i=0} for some subject i and \eqn{|t_i|=\infty}  and \eqn{p_i^{LOO}=0}. Then, these 
  features will be extremely significant but usually not of biological interest. To prevent these
  technical problems, the user may add a small positive constant \eqn{s_0} to \eqn{s_i}.
  The statistical significance of LOO-MOP and LOO-SST is calculated in the similar way to LMS-MOP and LMS-SST.}
 \examples{
 ## Not run
 # data(M7example)
 # head(M7example)
 # M7.pqt=pq.transform(M7example,5:18,F)
 # head(M7.pqt)
 # summary(M7.pqt)
 # output.folder=getwd()   # set the output folder
 # M7.res=full.oasis(M7example,
 #                   data.columns=5:18,
 #                   output.dir=output.folder,
 #                   plot.title.column=1,
 #                   ntop.plot=50,
 #                   nsim=100)
 # names(M7.res)
 # head(M7.res$oasis.res)
 # head(M7.res$sim.stats)
 }
